#!/usr/bin/perl

use strict;
use warnings;

use constant CLASS_MAJOR_VERSION_MAX => 50;

use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use Debian::Debhelper::Dh_Lib();

my %ALT_MAP = (
    'java6' => [ qw (java-runtime6) ],
    'java5' => [ map { "java-runtime$_" } (5, 6) ],
    'java2' => [ map { "java-runtime$_" } (2, 5, 6) ],
    'any' => [  map { "java-runtime$_" } ('', 2, 5, 6) ],
);

my $majv = find_highest_major_version(@ARGV);

sub generate_alternatives {
    my ($majv, $hl) = @_;
    my $java;
    my $alt = '';
    my $headless = '';
    $headless = '-headless' if ($hl);
    if ($majv < 48) {
        $java = 'any';
    } elsif ($majv < 49) {
        $java = 'java2';
    } elsif ($majv < 50) {
        $java = 'java5';
    } elsif ($majv < 51) {
        $java = 'java6';
    } else {
        error("Unknown major version $majv");
    }
    foreach my $a (@{ $ALT_MAP{$java} }){
        $alt = "$alt | $a$headless";
    }
    return $alt;
}

sub find_highest_major_version {
    my (@jars) = @_;
    my $majv = 0;
  JARS:
    foreach my $jar (@jars){
        my $zip = Archive::Zip->new();
        # stringify or $zip will make a call back that fails.
        $zip->read( $jar ) == AZ_OK or error("Could not read $jar: $!");
        foreach my $class ($zip->membersMatching('\.class$')){
            use bytes;
            my $classname = $class->fileName();
            my $status = $class->rewindData();
            my $buffref;
            my ($magic, $majver);
            # We would like to read it uncompressed.
            $class->desiredCompressionMethod( COMPRESSION_STORED );
            error("$classname (in $jar): $status")
                unless ($class->rewindData() == AZ_OK);
            ( $buffref, $status ) = $class->readChunk();
            error("Read of $classname (in $jar) failed: $status")
                unless ($status == AZ_OK || $status == AZ_STREAM_END);
            ($magic, $majver) = unpack('NN', $$buffref);
            printf "%s: %x %d\n",  $class->fileName(), $magic, $majver;
            $class->endRead();
            next unless ($magic == 0xCAFEBABE);
            if ($majver > $majv){
                $majv = $majver;
                last JARS if ($majv >= CLASS_MAJOR_VERSION_MAX);
            }
        }
    }
    return $majv;
}

if ($majv > CLASS_MAJOR_VERSION_MAX){
    error("Seen newer class version ($majv) that currently known.\n");
}

print "Highest class version seen $majv\n";
print ' - Alternatives: ' . generate_alternatives($majv) . "\n";
